@@ -1,13 +1,13 @@
 use std::collections::HashMap;
+use std::convert::TryInto;
 
-use crate::CacheManager;
+use crate::{CacheManager, Result};
 
 use http_cache_semantics::CachePolicy;
+use http_types::Version;
 use serde::{Deserialize, Serialize};
 use surf::{Request, Response};
 
-type Result<T> = std::result::Result<T, http_types::Error>;
-
 /// Implements [`CacheManager`] with [`cacache`](https://github.com/zkat/cacache-rs) as the backend.
 #[derive(Debug, Clone)]
 pub struct CACacheManager {
@@ -33,6 +33,8 @@ struct Store {
 struct StoredResponse {
     body: Vec<u8>,
     headers: HashMap<String, String>,
+    status: u16,
+    version: Version,
 }
 
 async fn to_store(res: &mut Response, policy: CachePolicy) -> Result<Store> {
@@ -40,22 +42,31 @@ async fn to_store(res: &mut Response, policy: CachePolicy) -> Result<Store> {
     for header in res.iter() {
         headers.insert(header.0.as_str().to_owned(), header.1.as_str().to_owned());
     }
+    let status = res.status().into();
+    let version = res.version().unwrap_or(Version::Http1_1);
     let body: Vec<u8> = res.body_bytes().await?;
     Ok(Store {
-        response: StoredResponse { body, headers },
+        response: StoredResponse {
+            body,
+            headers,
+            status,
+            version,
+        },
         policy,
     })
 }
 
-fn from_store(store: &Store) -> Response {
+fn from_store(store: &Store) -> Result<Response> {
     let mut res = http_types::Response::new(http_types::StatusCode::Ok);
     for header in &store.response.headers {
         let val =
             http_types::headers::HeaderValue::from_bytes(header.1.as_bytes().to_vec()).unwrap();
         res.insert_header(header.0.as_str(), val);
     }
+    res.set_status(store.response.status.try_into()?);
+    res.set_version(Some(store.response.version));
     res.set_body(store.response.body.clone());
-    Response::from(res)
+    Ok(Response::from(res))
 }
 
 fn req_key(req: &Request) -> String {
@@ -80,7 +91,7 @@ impl CacheManager for CACacheManager {
                 return Ok(None);
             }
         };
-        Ok(Some((from_store(&store), store.policy)))
+        Ok(Some((from_store(&store)?, store.policy)))
     }
 
     // TODO - This needs some reviewing.
@@ -98,6 +109,7 @@ impl CacheManager for CACacheManager {
         for header in res.iter() {
             ret_res.insert_header(header.0, header.1);
         }
+        ret_res.set_status(res.status());
         ret_res.set_version(res.version());
         Ok(Response::from(ret_res))
     }
@@ -122,7 +134,7 @@ mod tests {
         res.set_body("test");
         let mut res = surf::Response::from(res);
         let req = Request::new(Method::Get, url);
-        let policy = CachePolicy::new(&get_request_parts(&req), &get_response_parts(&res));
+        let policy = CachePolicy::new(&get_request_parts(&req)?, &get_response_parts(&res)?);
         let manager = CACacheManager::default();
         manager.put(&req, &mut res, policy).await?;
         let data = manager.get(&req).await?;
